@@ -39,6 +39,25 @@ def samefile(p1,p2):
         os.path.normpath(os.path.normcase(p2))
     )
 
+if sys.version_info <= (3,):
+    def _to_ascii(s):
+        return s
+    def isascii(s):
+        try:
+            unicode(s, 'ascii')
+            return True
+        except UnicodeError:
+            return False
+else:
+    def _to_ascii(s):
+        return s.encode('ascii')
+    def isascii(s):
+        try:
+            s.encode('ascii')
+            return True
+        except UnicodeError:
+            return False
+
 class easy_install(Command):
     """Manage a download/build/install process"""
     description = "Find/get/install Python packages"
@@ -71,11 +90,12 @@ class easy_install(Command):
         ('no-deps', 'N', "don't install dependencies"),
         ('allow-hosts=', 'H', "pattern(s) that hostnames must match"),
         ('local-snapshots-ok', 'l', "allow building eggs from local checkouts"),
+        ('version', None, "print version information and exit"),
     ]
     boolean_options = [
         'zip-ok', 'multi-version', 'exclude-scripts', 'upgrade', 'always-copy',
         'delete-conflicting', 'ignore-conflicts-at-my-risk', 'editable',
-        'no-deps', 'local-snapshots-ok',
+        'no-deps', 'local-snapshots-ok', 'version'
     ]
     negative_opt = {'always-unzip': 'zip-ok'}
     create_index = PackageIndex
@@ -91,6 +111,7 @@ class easy_install(Command):
         self.upgrade = self.always_copy = self.multi_version = None
         self.editable = self.no_deps = self.allow_hosts = None
         self.root = self.prefix = self.no_report = None
+        self.version = None
 
         # Options not specifiable via command line
         self.package_index = None
@@ -122,6 +143,10 @@ class easy_install(Command):
                         os.unlink(filename)
 
     def finalize_options(self):
+        if self.version:
+            print 'distribute %s' % get_distribution('distribute').version
+            sys.exit()
+
         self._expand('install_dir','script_dir','build_directory','site_dirs')
         # If a non-default installation directory was specified, default the
         # script directory to match it.
@@ -541,7 +566,7 @@ Please make the appropriate changes for your system and try again.
             return True
         if not dist.has_metadata('zip-safe'):
             return True
-        return False
+        return True
 
     def maybe_move(self, spec, dist_filename, setup_base):
         dst = os.path.join(self.build_directory, spec.key)
@@ -593,7 +618,7 @@ Please make the appropriate changes for your system and try again.
                 "import pkg_resources\n"
                 "pkg_resources.run_script(%(spec)r, %(script_name)r)\n"
             ) % locals()
-        self.write_script(script_name, script_text, 'b')
+        self.write_script(script_name, _to_ascii(script_text), 'b')
 
     def write_script(self, script_name, contents, mode="t", blockers=()):
         """Write an executable file to the scripts directory"""
@@ -1072,7 +1097,7 @@ Please make the appropriate changes for your system and try again.""" % (
             return  # already did it, or don't need to
 
         sitepy = os.path.join(self.install_dir, "site.py")
-        source = resource_string(Requirement.parse("setuptools"), "site.py")
+        source = resource_string(Requirement.parse("distribute"), "site.py")
         current = ""
 
         if os.path.exists(sitepy):
@@ -1186,7 +1211,12 @@ def get_site_dirs():
         site_lib = get_python_lib(plat_specific)
         if site_lib not in sitedirs: sitedirs.append(site_lib)
 
+    if sys.version >= "2.6":
+        import site
+        sitedirs.append(site.USER_SITE)
+
     sitedirs = map(normalize_path, sitedirs)
+
     return sitedirs
 
 
@@ -1375,7 +1405,7 @@ class PthDistributions(Environment):
 
             if os.path.islink(self.filename):
                 os.unlink(self.filename)
-            f = open(self.filename,'wb')
+            f = open(self.filename,'wt')
             f.write(data); f.close()
 
         elif os.path.exists(self.filename):
@@ -1426,7 +1456,7 @@ def get_script_header(script_text, executable=sys_executable, wininst=False):
     else:
         executable = nt_quote_arg(executable)
     hdr = "#!%(executable)s%(options)s\n" % locals()
-    if unicode(hdr,'ascii','ignore').encode('ascii') != hdr:
+    if not isascii(hdr):
         # Non-ascii path to sys.executable, use -x to prevent warnings
         if options:
             if options.strip().startswith('-'):
@@ -1566,16 +1596,18 @@ def get_script_args(dist, executable=sys_executable, wininst=False):
     spec = str(dist.as_requirement())
     header = get_script_header("", executable, wininst)
     for group in 'console_scripts', 'gui_scripts':
-        for name,ep in dist.get_entry_map(group).items():
+        for name, ep in dist.get_entry_map(group).items():
             script_text = (
                 "# EASY-INSTALL-ENTRY-SCRIPT: %(spec)r,%(group)r,%(name)r\n"
                 "__requires__ = %(spec)r\n"
                 "import sys\n"
                 "from pkg_resources import load_entry_point\n"
                 "\n"
-                "sys.exit(\n"
-                "   load_entry_point(%(spec)r, %(group)r, %(name)r)()\n"
-                ")\n"
+                "if __name__ == '__main__':"
+                "\n"
+                "    sys.exit(\n"
+                "        load_entry_point(%(spec)r, %(group)r, %(name)r)()\n"
+                "    )\n"
             ) % locals()
             if sys.platform=='win32' or wininst:
                 # On Windows/wininst, add a .py extension and an .exe launcher
@@ -1642,7 +1674,6 @@ def bootstrap():
     import setuptools; argv0 = os.path.dirname(setuptools.__path__[0])
     sys.argv[0] = argv0; sys.argv.append(argv0); main()
 
-
 def main(argv=None, **kw):
     from setuptools import setup
     from setuptools.dist import Distribution
@@ -1667,9 +1698,16 @@ usage: %(script)s [options] requirement_or_url ...
 
     class DistributionWithoutHelpCommands(Distribution):
         common_usage = ""
+
         def _show_help(self,*args,**kw):
             with_ei_usage(lambda: Distribution._show_help(self,*args,**kw))
 
+        def find_config_files(self):
+            files = Distribution.find_config_files(self)
+            if 'setup.cfg' in files:
+                files.remove('setup.cfg')
+            return files
+
     if argv is None:
         argv = sys.argv[1:]
 
